@@ -34,6 +34,7 @@
 #include "kz-favicon.h"
 #include "kz-xml.h"
 #include "kz-actions-dynamic.h"
+#include "kz-icons.h"
 
 #define KZ_ACTIONS_POPUP_LANGUAGE_KEY "KzActionsPopup::Language"
 #define KZ_ACTIONS_POPUP_TAB_KEY "KzActionsPopup::Tab"
@@ -696,7 +697,7 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 	GtkWidget *tablist_submenu;
 	GtkWidget *page;
 	gint current_page_num;
-	KzFavicon *kzfav;
+	KzFavicon *kzfav = kz->kzfav;
 	gchar *color = NULL;
 
 	/* label color */
@@ -725,8 +726,6 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 	label_color[KZ_TAB_LABEL_STATE_LOADED] = g_strdup(color);
 	g_free(color);
 
-	kzfav = kz_favicon_get_instance();
-
 	if (GTK_IS_MENU_ITEM(tablist_menu))
 	{
 		if (!popup_menu_table)
@@ -822,7 +821,6 @@ kz_actions_popup_append_tablist_menuitem (KzWindow *kz, GtkWidget *tablist_menu)
 		g_signal_connect(tablist_menu, "destroy",
 				 G_CALLBACK(cb_popup_destroy), NULL);
 	}
-	g_object_unref(kzfav);
 	return;
 }
 
