@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-editor.c,v 1.40 2005/02/14 05:45:04 ikezoe Exp $
+ *  $Id: kz-bookmark-editor.c,v 1.41 2005/05/09 01:20:12 ikezoe Exp $
  */
 
 #include "kz-bookmark-editor.h"
@@ -83,7 +83,6 @@ static GtkWindowClass *parent_class = NULL;
 KZ_OBJECT_GET_TYPE(kz_bookmark_editor, "KzBookmarkEditor", KzBookmarkEditor,
 		   kz_bookmark_editor_class_init, kz_bookmark_editor_init,
 		   GTK_TYPE_WINDOW)
-KZ_OBJECT_FINALIZE(kz_bookmark_editor, KzBookmarkEditor)
 
 
 static void
@@ -98,7 +97,6 @@ kz_bookmark_editor_class_init (KzBookmarkEditorClass *klass)
 	widget_class  = (GtkWidgetClass *) klass;
 
 	gobject_class->dispose      = kz_bookmark_editor_dispose;
-	gobject_class->finalize     = kz_bookmark_editor_finalize;
 	gobject_class->set_property = kz_bookmark_editor_set_property;
 	gobject_class->get_property = kz_bookmark_editor_get_property;
 
@@ -249,8 +247,6 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	editor->action_group   = actions;
 	editor->menu_merge     = menu_merge;
 
-	editor->priv = NULL;
-
 	/* set default state */
 	kz_bookmark_editor_restore_state(editor);
 
