@@ -18,7 +18,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.c,v 1.76 2005/02/13 22:55:15 ikezoe Exp $
+ *  $Id: kz-bookmark.c,v 1.77 2005/07/13 05:37:06 ikezoe Exp $
  */
 
 #include "kz-bookmark.h"
@@ -770,14 +770,15 @@ kz_bookmark_real_insert_child (KzBookmark *bookmark,
 			       KzBookmark *child,
 			       KzBookmark *sibling)
 {
-	GList *list, *next;
+	GList *list, *next = NULL;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(child));
 	g_return_if_fail(!sibling || KZ_IS_BOOKMARK(sibling));
 	g_return_if_fail(kz_bookmark_is_folder(bookmark));
 
 	list = g_object_get_qdata(G_OBJECT(bookmark), children_quark);
-	next = g_list_find(list, sibling);
+	if (sibling)
+		next = g_list_find(list, sibling);
 
 	g_object_ref(child);
 	list = g_list_insert_before(list, next, child);
