@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c,v 1.61 2005/02/13 13:20:34 ikezoe Exp $
+ *  $Id: kz-xbel.c,v 1.64 2005/07/13 00:14:55 ikezoe Exp $
  */
 
 #include "kazehakase.h"
@@ -63,6 +63,7 @@ static void     xml_node_set_title         (KzXMLNode   *parent,
 
 static GQuark xml_quark = 0;
 static GQuark node_quark = 0;
+static GQuark root_quark = 0;
 static GQuark building_quark = 0;
 
 
@@ -125,6 +126,8 @@ kz_xbel_init (KzBookmarkFile *bookmark)
 
 	if (!xml_quark)
 		xml_quark = g_quark_from_string("KzXBEL::KzXML");
+	if (!root_quark)
+		root_quark = g_quark_from_string("KzXBEL::XMLRootNode");
 	if (!node_quark)
 		node_quark = g_quark_from_string("KzXBEL::XMLNode");
 	if (!building_quark)
@@ -288,6 +291,7 @@ static void
 parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 {
 	KzXMLNode *node;
+	KzBookmark *smart_history = NULL;
 
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
@@ -304,6 +308,7 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 		const gchar *smart_encode = NULL;
 		gboolean urlencode = FALSE;
 		gint interval;
+		gboolean is_smart_history = FALSE;
 
 		if (!kz_xml_node_is_element(node)) continue;
 
@@ -337,10 +342,16 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 			{
 				smart_encode = attr->value;
 			}
-			else if (!strncmp(attr->name, "kz:smart_urlencode", 16))
+			else if (!strncmp(attr->name, "kz:smart_urlencode", 18))
 			{
 				urlencode = TRUE;
 			}
+			else if (!strncmp(attr->name, "kz:smart_history", 16) && !smart_history)
+			{
+				smart_history = KZ_BOOKMARK(kz_bookmark_file_new(attr->value,
+							    NULL, NULL));
+				is_smart_history = TRUE;
+			}
 			else if (!strncmp(attr->name, "kz:xmlrpc", 9))
 			{
 				kz_bookmark_file_set_xmlrpc(KZ_BOOKMARK_FILE(bookmark), attr->value);
@@ -353,6 +364,18 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 			{
 				kz_bookmark_file_set_xmlrpc_pass(KZ_BOOKMARK_FILE(bookmark), attr->value);
 			}
+			else if (!strncmp(attr->name, "kz:smart_xmlrpc", 9))
+			{
+				kz_bookmark_file_set_xmlrpc(KZ_BOOKMARK_FILE(smart_history), attr->value);
+			}
+			else if (!strncmp(attr->name, "kz:smart_xmlrpc_user", 14))
+			{
+				kz_bookmark_file_set_xmlrpc_user(KZ_BOOKMARK_FILE(smart_history), attr->value);
+			}
+			else if (!strncmp(attr->name, "kz:smart_xmlrpc_pass", 14))
+			{
+				kz_bookmark_file_set_xmlrpc_pass(KZ_BOOKMARK_FILE(smart_history), attr->value);
+			}
 			else if (!strncmp(attr->name, "kz:current_position", 18))
 			{
 				gint pos = atoi(attr->value);
@@ -368,7 +391,6 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 				if (!strcmp(attr->value, "yes"))
 					kz_bookmark_set_auto_refresh(bookmark, TRUE);
 			}
-
 		}
 
 		if (smart_regex && smart_uri)
@@ -378,7 +400,15 @@ parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
 				smart_regex, smart_uri, 
 				smart_encode, urlencode);
 		}
-	}	
+	}
+
+	if (smart_history)
+	{
+		kz_bookmark_file_load_start(KZ_BOOKMARK_FILE(smart_history));
+		kz_smart_bookmark_set_history (KZ_SMART_BOOKMARK(bookmark),
+					       smart_history);
+		g_object_unref(smart_history);
+	}
 }
 
 static KzXMLNode *
@@ -663,12 +693,36 @@ kz_xbel_build_tree (KzBookmark *bookmark)
 
 	BOOKMARK_SET_BUILDING(bookmark);
 	g_object_set_qdata(G_OBJECT(bookmark),
-			   node_quark, node);
+			   root_quark, node);
 	parse_child_node(bookmark, node);
 	BOOKMARK_UNSET_BUILDING(bookmark);
 }
 
 
+static KzXMLNode *
+xml_node_ensure_info_node (KzXMLNode *node)
+{
+	KzXMLNode *info_node, *space;
+
+	info_node = xml_node_get_named_node(node, "info");
+
+	if (info_node)
+		return info_node;
+		
+	info_node = kz_xml_element_node_new("info");
+	kz_xml_node_append_child(node, info_node);
+
+		/* space1 */
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(info_node, space);
+
+		/* space4 */
+	space = kz_xml_text_node_new("\n");
+	kz_xml_node_append_child(node, space);
+
+	return info_node;
+}
+
 static void
 xml_node_append_title (KzXMLNode *node, KzBookmark *bookmark)
 {
@@ -704,20 +758,7 @@ xml_node_append_metadata_node (KzXMLNode *node)
 	 */
 
 	/* info */
-	info_node = xml_node_get_named_node(node, "info");
-	if (!info_node)
-	{
-		info_node = kz_xml_element_node_new("info");
-		kz_xml_node_append_child(node, info_node);
-
-		/* space1 */
-		space = kz_xml_text_node_new("\n");
-		kz_xml_node_append_child(info_node, space);
-
-		/* space4 */
-		space = kz_xml_text_node_new("\n");
-		kz_xml_node_append_child(node, space);
-	}
+	info_node = xml_node_ensure_info_node(node);
 
 	/* metadata */
 	metadata_node = xml_node_find_metadata_node(node);
@@ -782,22 +823,7 @@ xml_node_set_smart_list (KzXMLNode *node,
 	KzXMLNode *info_node, *child_node;
 
 	/* get the info node, create it if not exist */
-	info_node = xml_node_get_named_node(node, "info");
-	if (!info_node)
-	{
-		KzXMLNode *space;
-
-		info_node = kz_xml_element_node_new("info");
-		kz_xml_node_append_child(node, info_node);
-
-		/* space1 */
-		space = kz_xml_text_node_new("\n");
-		kz_xml_node_append_child(info_node, space);
-
-		/* space4 */
-		space = kz_xml_text_node_new("\n");
-		kz_xml_node_append_child(node, space);
-	}
+	info_node = xml_node_ensure_info_node(node);
 
 	/* remove all previous smart properties */
 	child_node = kz_xml_node_first_child(info_node);
@@ -876,6 +902,85 @@ xml_node_set_smart_list (KzXMLNode *node,
 }
 
 
+static void
+xml_node_set_smart_history (KzXMLNode *node,
+			    KzBookmark *bookmark,
+		       	    KzBookmark *smart_history)
+{
+	const gchar *history_location = NULL;
+	const gchar *xmlrpc, *xml_user, *xml_pass;
+	KzXMLNode *info_node, *child_node, *smart_history_node = NULL;
+
+	if (!KZ_IS_BOOKMARK_FILE(smart_history))
+		return;
+
+	history_location = kz_bookmark_file_get_location(
+					KZ_BOOKMARK_FILE(smart_history));
+
+	xmlrpc           = kz_bookmark_file_get_xmlrpc(
+					KZ_BOOKMARK_FILE(smart_history));
+
+	xml_user         = kz_bookmark_file_get_xmlrpc_user(
+					KZ_BOOKMARK_FILE(smart_history));
+
+	xml_pass         = kz_bookmark_file_get_xmlrpc_pass(
+					KZ_BOOKMARK_FILE(smart_history));
+
+	/* get the info node, create it if not exist */
+	info_node = xml_node_ensure_info_node(node);
+	
+	for (child_node = kz_xml_node_first_child(info_node);
+	     child_node;
+	     child_node = kz_xml_node_next(child_node))
+	{
+		const gchar *owner;
+		const gchar *tmp;
+	
+		if (!kz_xml_node_name_is(child_node, "metadata"))
+			continue;
+
+		owner = kz_xml_node_get_attr(child_node, "owner");
+		if (!owner || strcmp(owner, KAZEHAKASE_URI))
+			continue;
+
+		tmp = kz_xml_node_get_attr(child_node, "kz:smart_history");
+
+		if (tmp)
+		{
+			smart_history_node = child_node;
+			break;
+		}
+	}
+
+	if (!smart_history_node)
+	{
+		smart_history_node = kz_xml_element_node_new("metadata");
+		kz_xml_node_set_attr(smart_history_node, "owner", KAZEHAKASE_URI);
+		kz_xml_node_append_child(info_node, smart_history_node);
+	}
+
+	kz_xml_node_set_attr(smart_history_node,
+			     "kz:smart_history", history_location);
+
+
+	if (xmlrpc)
+	{
+		kz_xml_node_set_attr(smart_history_node,
+				     "kz:smart_xmlrpc", xmlrpc);
+	}
+	if (xml_user)
+	{
+		kz_xml_node_set_attr(smart_history_node,
+				     "kz:smart_xmlrpc_user", xml_user);
+	}
+	if (xml_pass)
+	{
+		kz_xml_node_set_attr(smart_history_node,
+				     "kz:smart_xmlrpc_pass", xml_pass);
+	}
+}
+
+
 static KzXMLNode *
 create_xml_node (KzBookmark *bookmark)
 {
@@ -1039,14 +1144,12 @@ kz_xbel_connect_signals (KzBookmark *bookmark)
 	if (KZ_IS_BOOKMARK_FILE(bookmark)) return;
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
-	{
-		g_signal_connect_after(bookmark, "insert-child",
-				       G_CALLBACK(cb_bookmark_insert_child),
-				       NULL);
-		g_signal_connect_after(bookmark, "remove-child",
-				       G_CALLBACK(cb_bookmark_remove_child),
-				       NULL);
-	}
+	g_signal_connect_after(bookmark, "insert-child",
+			       G_CALLBACK(cb_bookmark_insert_child),
+			       NULL);
+	g_signal_connect_after(bookmark, "remove-child",
+			       G_CALLBACK(cb_bookmark_remove_child),
+			       NULL);
 
 	/* for children */
 	{
@@ -1073,14 +1176,12 @@ kz_xbel_disconnect_signals (KzBookmark *bookmark)
 	if (KZ_IS_BOOKMARK_FILE(bookmark)) return;
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
-	{
-		g_signal_handlers_disconnect_by_func
-			(bookmark,
-			 G_CALLBACK(cb_bookmark_insert_child), NULL);
-		g_signal_handlers_disconnect_by_func
-			(bookmark,
-			 G_CALLBACK(cb_bookmark_remove_child), NULL);
-	}
+	g_signal_handlers_disconnect_by_func
+		(bookmark,
+		 G_CALLBACK(cb_bookmark_insert_child), NULL);
+	g_signal_handlers_disconnect_by_func
+		(bookmark,
+		 G_CALLBACK(cb_bookmark_remove_child), NULL);
 
 	/* for children */
 	{
@@ -1284,6 +1385,12 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 		smart_list =  g_value_get_pointer(&value);
 		xml_node_set_smart_list(node, bookmark, smart_list);
 	}
+	else if (!strcmp(prop, "smart-history"))
+	{
+		KzBookmark *smart_history;
+		smart_history =  g_value_get_object(&value);
+		xml_node_set_smart_history(node, bookmark, smart_history);
+	}
 	else if (!strcmp(prop, "xmlrpc"))
 	{
 		KzXMLNode *metanode;
