@@ -26,9 +26,22 @@
 #include "gobject-utils.h"
 #include "gtk-utils.h"
 
-static void kz_download_box_class_init (KzDownloadBoxClass *klass);
-static void kz_download_box_init       (KzDownloadBox      *dlbox);
-static void kz_download_box_dispose    (GObject            *object);
+enum {
+	PROP_0,
+	PROP_KZ_WINDOW
+};
+
+static void kz_download_box_class_init   (KzDownloadBoxClass *klass);
+static void kz_download_box_init         (KzDownloadBox      *dlbox);
+static void kz_download_box_dispose      (GObject            *object);
+static void kz_download_box_set_property (GObject            *object,
+					  guint               prop_id,
+					  const GValue       *value,
+					  GParamSpec         *pspec);
+static void kz_download_box_get_property (GObject            *object,
+					  guint               prop_id,
+					  GValue             *value,
+					  GParamSpec         *pspec);
 
 static void kz_download_box_drag_data_received (GtkWidget        *widget,
 						GdkDragContext   *context,
@@ -80,13 +93,26 @@ kz_download_box_class_init (KzDownloadBoxClass *klass)
 	gobject_class = (GObjectClass *) klass;
 	widget_class  = (GtkWidgetClass *) klass;
 
-	/* GtkObject signals */
-	gobject_class->dispose = kz_download_box_dispose;
+	/* GObject signals */
+	gobject_class->dispose      = kz_download_box_dispose;
+	gobject_class->set_property = kz_download_box_set_property;
+	gobject_class->get_property = kz_download_box_get_property;
 
 	/* GtkWidget signales */
 	/* widget_class->drag_motion          = kz_download_box_drag_motion; */
 	/* widget_class->drag_data_get        = kz_download_box_drag_data_get; */
 	widget_class->drag_data_received   = kz_download_box_drag_data_received;
+	
+	g_object_class_install_property
+		(gobject_class,
+		 PROP_KZ_WINDOW,
+		 g_param_spec_object ("kz-window",
+				      _("KzWindow"),
+				      _("The parent kazehakase window"),
+				      KZ_TYPE_WINDOW,
+				      G_PARAM_READWRITE |
+				      G_PARAM_CONSTRUCT_ONLY));
+
 }
 
 static void
@@ -95,7 +121,7 @@ kz_download_box_init (KzDownloadBox *dlbox)
 	/* widgets */
 	dlbox->kz = NULL;
 	
-	dlbox->dlgrp     = NULL;
+	dlbox->dlgrp = kz_downloader_group_get_instance();
 	dlbox->icon_list = NULL;
 
 	gtk_drag_dest_set(GTK_WIDGET(dlbox), 
@@ -103,27 +129,63 @@ kz_download_box_init (KzDownloadBox *dlbox)
 			  GTK_DEST_DEFAULT_DROP,
                           url_drag_types,n_url_drag_types,
 			  GDK_ACTION_LINK);
+	
+	g_signal_connect(dlbox->dlgrp, "add",
+			 G_CALLBACK(cb_add_item), dlbox);
+	g_signal_connect(dlbox->dlgrp, "remove",
+			 G_CALLBACK(cb_remove_item), dlbox);
+	g_signal_connect(dlbox->dlgrp, "progress",
+			 G_CALLBACK(cb_progress_item), dlbox);
 }
 
 
+static void
+kz_download_box_set_property (GObject            *object,
+			      guint               prop_id,
+			      const GValue       *value,
+			      GParamSpec         *pspec)
+{
+	KzDownloadBox *dlbox = KZ_DOWNLOAD_BOX(object);
+  
+	switch (prop_id)
+	{
+	case PROP_KZ_WINDOW:
+		dlbox->kz = g_object_ref(g_value_get_object(value));
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+kz_download_box_get_property (GObject            *object,
+			      guint               prop_id,
+			      GValue             *value,
+			      GParamSpec         *pspec)
+{
+	KzDownloadBox *dlbox = KZ_DOWNLOAD_BOX(object);
+
+	switch (prop_id)
+	{
+	case PROP_KZ_WINDOW:
+		g_value_set_object(value, dlbox->kz);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 GtkWidget *
 kz_download_box_new (KzWindow *kz)
 {
 	KzDownloadBox *dlbox;
 
-	dlbox = g_object_new(KZ_TYPE_DOWNLOAD_BOX, NULL);
-
-	dlbox->kz = kz;
+	dlbox = g_object_new(KZ_TYPE_DOWNLOAD_BOX,
+		       	     "kz-window", kz,
+			     NULL);
 	
-	dlbox->dlgrp = kz_downloader_group_get_instance();
-
-	g_signal_connect(dlbox->dlgrp, "add",
-			 G_CALLBACK(cb_add_item), dlbox);
-	g_signal_connect(dlbox->dlgrp, "remove",
-			 G_CALLBACK(cb_remove_item), dlbox);	
-	g_signal_connect(dlbox->dlgrp, "progress",
-			 G_CALLBACK(cb_progress_item), dlbox);	
-
 	return GTK_WIDGET(dlbox);
 }
 
@@ -133,6 +195,12 @@ kz_download_box_dispose (GObject *object)
 {
 	KzDownloadBox *dlbox = KZ_DOWNLOAD_BOX(object);
 
+	if (dlbox->kz)
+	{
+		g_object_unref(dlbox->kz);
+		dlbox->kz = NULL;
+	}
+
 	if (dlbox->icon_list)
 	{
 		g_slist_free(dlbox->icon_list);
@@ -211,14 +279,49 @@ kz_download_box_drag_data_received (GtkWidget *widget,
 
 
 static void
-cb_icon_clicked (GtkWidget *event_box,
-		 GdkEventButton *button,
-		 KzDownloaderIcon *icon)
+cb_download_icon_destroy (GtkWidget *widget, GtkWidget *menu)
+{
+	gtk_widget_destroy (menu);
+}
+
+
+static void 
+cb_cancel_menu_activate (GtkWidget *widget, KzDownloaderIcon *icon)
 {
 	kz_downloader_cancel (icon->dl);
 }
 
 
+static void
+cb_icon_clicked (GtkWidget *event_box,
+		 GdkEventButton *event,
+		 KzDownloaderIcon *icon)
+{
+	GtkWidget *menu, *menuitem;
+
+	switch (event->button) {
+	case 1:
+		break;
+	case 2:
+		break;
+	case 3:
+		menu = gtk_menu_new();
+		g_signal_connect(G_OBJECT(icon->event_box), "destroy",
+				 G_CALLBACK(cb_download_icon_destroy), menu);
+		menuitem = gtk_image_menu_item_new_from_stock(GTK_STOCK_CANCEL, NULL);
+		g_signal_connect(G_OBJECT(menuitem), "activate",
+				 G_CALLBACK(cb_cancel_menu_activate), icon);
+		gtk_menu_shell_append(GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_show (menuitem);
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
+				event->button, event->time);
+		break;
+	default:
+		break;
+	}
+}
+
+
 static void
 cb_add_item (KzDownloaderGroup *dlgrp,
 	     KzDownloader      *dl,
