@@ -53,10 +53,12 @@ enum {
 	N_COLUMN
 };
 
-struct _KzPrefsWinPriv
+typedef struct _KzPrefsWinPrivate       KzPrefsWinPrivate;
+struct _KzPrefsWinPrivate
 {
 	GList *page_list;
 };
+#define KZ_PREFS_WIN_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), KZ_TYPE_PREFS_WIN, KzPrefsWinPrivate))
 
 typedef struct _KzPrefsWinPagePriv
 {
@@ -68,28 +70,27 @@ typedef struct _KzPrefsWinPagePriv
 static void       kz_prefs_win_class_init        (KzPrefsWinClass *klass);
 static void       kz_prefs_win_init              (KzPrefsWin      *win);
 static void       kz_prefs_win_dispose           (GObject         *object);
-static void       kz_prefs_win_finalize          (GObject         *finalize);
 static void       kz_prefs_win_response          (GtkDialog       *dialog,
 						  gint             arg);
-static gboolean   kz_prefs_win_delete_event      (GtkWidget        *widget,
-						  GdkEventAny      *event);
+static gboolean   kz_prefs_win_delete_event      (GtkWidget       *widget,
+						  GdkEventAny     *event);
 
-static GtkWidget *kz_prefs_win_create_nav_tree   (KzPrefsWin *win);
-static void       kz_prefs_win_reset_nav_tree    (KzPrefsWin *win);
-static void       kz_prefs_win_set_ui_level      (KzPrefsWin *win);
-static void       kz_prefs_win_restore_state     (KzPrefsWin *win);
-static void       kz_prefs_win_store_state       (KzPrefsWin *win);
+static GtkWidget *kz_prefs_win_create_nav_tree   (KzPrefsWin      *win);
+static void       kz_prefs_win_reset_nav_tree    (KzPrefsWin      *win);
+static void       kz_prefs_win_set_ui_level      (KzPrefsWin      *win);
+static void       kz_prefs_win_restore_state     (KzPrefsWin      *win);
+static void       kz_prefs_win_store_state       (KzPrefsWin      *win);
 
-static void       cb_profile_global_changed      (KzProfile   *profile,
-						  const gchar *section,
-						  const gchar *key,
-						  const gchar *old_value,
-						  KzPrefsWin  *win);
-static gboolean   cb_tree_key_press              (GtkWidget *widget,
-						  GdkEventKey *event,
-						  gpointer data);
-static void       cb_tree_cursor_changed         (GtkTreeView *treeview,
-						  gpointer data);
+static void       cb_profile_global_changed      (KzProfile       *profile,
+						  const gchar     *section,
+						  const gchar     *key,
+						  const gchar     *old_value,
+						  KzPrefsWin      *win);
+static gboolean   cb_tree_key_press              (GtkWidget       *widget,
+						  GdkEventKey     *event,
+						  gpointer         data);
+static void       cb_tree_cursor_changed         (GtkTreeView     *treeview,
+						  gpointer         data);
 
 
 static KzPrefsWin *kz_prefs_win_single = NULL;
@@ -111,8 +112,6 @@ static GList *prefs_page_entries = NULL;
 KZ_OBJECT_GET_TYPE(kz_prefs_win, "KzPrefsWin", KzPrefsWin,
 		   kz_prefs_win_class_init, kz_prefs_win_init,
 		   GTK_TYPE_DIALOG)
-KZ_OBJECT_FINALIZE(kz_prefs_win, KzPrefsWin)
-
 
 static void
 kz_prefs_win_class_init (KzPrefsWinClass *klass)
@@ -127,7 +126,6 @@ kz_prefs_win_class_init (KzPrefsWinClass *klass)
 	widget_class = (GtkWidgetClass *) klass;
 
 	object_class->dispose      = kz_prefs_win_dispose;
-	object_class->finalize     = kz_prefs_win_finalize;
 
 	dialog_class->response     = kz_prefs_win_response;
 	
@@ -146,6 +144,8 @@ kz_prefs_win_class_init (KzPrefsWinClass *klass)
 	/* ADD_PAGE_ENTRY(prefs_ui_editor_get_entry); */
 	ADD_PAGE_ENTRY(prefs_external_program_get_entry);
 	ADD_PAGE_ENTRY(prefs_history_get_entry);
+	
+	g_type_class_add_private (object_class, sizeof(KzPrefsWinPrivate));
 }
 
 
@@ -153,9 +153,9 @@ static void
 kz_prefs_win_init (KzPrefsWin *win)
 {
 	GtkWidget *paned, *scrwin, *tree, *notebook;
+	KzPrefsWinPrivate *priv = KZ_PREFS_WIN_GET_PRIVATE (win);
 
-	win->priv = g_new0(KzPrefsWinPriv, 1);
-	win->priv->page_list   = NULL;
+	priv->page_list   = NULL;
 
 	gtk_window_set_default_size(GTK_WINDOW(win), 600, 450);
 	gtk_window_set_icon(GTK_WINDOW(win), kz_icon);
@@ -176,7 +176,8 @@ kz_prefs_win_init (KzPrefsWin *win)
 	/* scrolled window */
 	scrwin = gtk_scrolled_window_new(NULL, NULL);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwin),
-				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+				       GTK_POLICY_AUTOMATIC,
+				       GTK_POLICY_AUTOMATIC);
         gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwin),
 					    GTK_SHADOW_IN);
 	gtk_widget_set_size_request(scrwin, 170, -1);
@@ -218,16 +219,17 @@ static void
 kz_prefs_win_dispose (GObject *object)
 {
 	KzPrefsWin *win = KZ_PREFS_WIN(object);
+	KzPrefsWinPrivate *priv = KZ_PREFS_WIN_GET_PRIVATE (object);
 
 	g_signal_handlers_disconnect_by_func
 		(kz_global_profile,
 		 G_CALLBACK(cb_profile_global_changed), win);
 
-	if (win->priv->page_list)
+	if (priv->page_list)
 	{
-		g_list_foreach (win->priv->page_list, (GFunc) g_free, NULL);
-		g_list_free(win->priv->page_list);
-		win->priv->page_list = NULL;
+		g_list_foreach (priv->page_list, (GFunc) g_free, NULL);
+		g_list_free(priv->page_list);
+		priv->page_list = NULL;
 	}
 
 	if (kz_prefs_win_single && kz_prefs_win_single == win)
@@ -241,7 +243,8 @@ kz_prefs_win_dispose (GObject *object)
 static void
 kz_prefs_win_response (GtkDialog *dialog, gint arg)
 {
-	GList *node = KZ_PREFS_WIN(dialog)->priv->page_list;
+	KzPrefsWinPrivate *priv = KZ_PREFS_WIN_GET_PRIVATE (dialog);
+	GList *node = priv->page_list;
 
 	for (; node; node = g_list_next(node))
 	{
@@ -321,7 +324,8 @@ kz_prefs_win_create_page (KzPrefsWin *win, KzPrefsWinPagePriv *priv)
 	if (priv->entry->create) {
 		widget = priv->entry->create();
 		label = gtk_label_new(title);
-		gtk_notebook_append_page(GTK_NOTEBOOK(win->notebook), widget, label);
+		gtk_notebook_append_page(GTK_NOTEBOOK(win->notebook),
+					 widget, label);
 		priv->widget = widget;
 		kz_prefs_win_set_ui_level(win);
 	}
@@ -335,15 +339,16 @@ kz_prefs_win_set_page (KzPrefsWin *win, const gchar *path)
 	KzPrefsWinPagePriv *priv = NULL;
 	GList *node;
 	gint num;
+	KzPrefsWinPrivate *winpriv = KZ_PREFS_WIN_GET_PRIVATE (win);
 
 	if (!path || !*path)
 	{
-		if (!win->priv->page_list) return;
-		priv = win->priv->page_list->data;
+		if (!winpriv->page_list) return;
+		priv = winpriv->page_list->data;
 	}
 	else
 	{
-		for (node = win->priv->page_list; node; node = g_list_next(node))
+		for (node = winpriv->page_list; node; node = g_list_next(node))
 		{
 			priv = node->data;
 			if (priv->entry && !strcmp(path, priv->entry->path)) 
@@ -354,8 +359,8 @@ kz_prefs_win_set_page (KzPrefsWin *win, const gchar *path)
 
 	if (!priv)
 	{
-		if (!win->priv->page_list) return;
-		priv = win->priv->page_list->data;
+		if (!winpriv->page_list) return;
+		priv = winpriv->page_list->data;
 	}
 
 	if (!priv->widget)
@@ -380,7 +385,6 @@ kz_prefs_win_create_nav_tree (KzPrefsWin *win)
 	GtkTreeStore *store;
 	GtkTreeViewColumn *col;
 	GtkCellRenderer *render;
-	GList *node;
 
 	store = gtk_tree_store_new(N_COLUMN,
 				   GDK_TYPE_PIXBUF, GDK_TYPE_PIXBUF,
@@ -416,46 +420,6 @@ kz_prefs_win_create_nav_tree (KzPrefsWin *win)
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (tree), col);
 	gtk_tree_view_set_expander_column (GTK_TREE_VIEW (tree), col);
-
-	/* set tree */
-	gtk_tree_store_clear(store);
-	for (node = prefs_page_entries; node; node = g_list_next(node))
-	{
-		KzPrefsWinPagePriv *priv;
-		KzPrefsWinPageEntry *entry = node->data;
-		GtkTreeIter iter;
-		gchar *title;
-
-		/* set private data */
-		priv = g_new0(KzPrefsWinPagePriv, 1);
-		priv->entry  = entry;
-		priv->widget = NULL;
-		win->priv->page_list = g_list_append (win->priv->page_list, priv);
-
-#if 0
-		if (kz_prefs_win_ensure_parent (priv, &parent_iter))
-		{
-			gtk_tree_store_append (store, &iter, &parent_iter);
-		}
-		else
-		{
-			gtk_tree_store_append (store, &iter, NULL);
-		}
-#else
-		gtk_tree_store_append (store, &iter, NULL);
-#endif
-
-		title = g_path_get_basename(_(entry->path));
-		gtk_tree_store_set(store, &iter,
-#if 0
-				   COLUMN_ICON_CLOSE,      pixbuf,
-				   COLUMN_ICON_OPEN,       pixbuf_open,
-#endif
-				   COLUMN_NAME,            title,
-				   COLUMN_PRIV_DATA,       priv,
-				   COLUMN_TERMINATOR);
-		g_free(title);
-	}
 	
 	g_object_unref(store);
 	return tree;
@@ -472,29 +436,78 @@ compare_page_entry (gconstpointer a, gconstpointer b)
 }
 
 
+/* FIXME! create parent if it doesn't exist */
+static gboolean
+kz_prefs_win_ensure_parent (KzPrefsWin *win,
+			    KzPrefsWinPagePriv *priv,
+			    GtkTreeIter *parent_iter)
+{
+	GtkTreeView *treeview = GTK_TREE_VIEW (win->tree_view);
+	GtkTreeModel *model = gtk_tree_view_get_model (treeview);
+	GtkTreeIter iter;
+	gchar *dir = g_path_get_dirname (priv->entry->path);
+	gboolean exist;
+
+	if (!dir)
+		return FALSE;
+	if (!*dir) {
+		g_free (dir);
+		return FALSE;
+	}
+
+	for (exist = gtk_tree_model_get_iter_first (model, &iter);
+	     exist;
+	     exist = gtk_tree_model_iter_next (model, &iter))
+	{
+		KzPrefsWinPagePriv *parent_priv = NULL;
+
+		gtk_tree_model_get (model, &iter,
+				    COLUMN_PRIV_DATA, &parent_priv,
+				    -1);
+		if (parent_priv &&
+		    parent_priv->entry &&
+		    parent_priv->entry->path &&
+		    !strcmp (dir, parent_priv->entry->path))
+		{
+			if (parent_iter)
+				*parent_iter = iter;
+			g_free (dir);
+			return TRUE;
+		}
+	}
+
+	g_free (dir);
+
+	return FALSE;
+}
+
 static void
 kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 {
 	GtkTreeView *treeview;
 	GtkTreeStore *store;
+	GtkTreePath *path;
 	GList *node;
+	KzPrefsWinPrivate *winpriv;
 
 	g_return_if_fail(KZ_IS_PREFS_WIN(win));
 
+	winpriv = KZ_PREFS_WIN_GET_PRIVATE (win);
 	treeview = GTK_TREE_VIEW(win->tree_view);
 	store = GTK_TREE_STORE(gtk_tree_view_get_model(treeview));
+	gtk_tree_view_get_cursor(treeview, &path, NULL);
 
 	gtk_tree_store_clear(store);
 	for (node = prefs_page_entries; node; node = g_list_next(node))
 	{
 		KzPrefsWinPagePriv *priv;
 		KzPrefsWinPageEntry *entry = node->data;
-		GtkTreeIter iter;
+		GtkTreeIter iter, parent_iter;
 		gchar *title;
 		GList *node;
 
 		/* set private data */
-		node = g_list_find_custom(win->priv->page_list, entry,
+		node = g_list_find_custom(winpriv->page_list, entry,
 					  compare_page_entry);
 		if (node)
 		{
@@ -505,7 +518,8 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 			priv = g_new0(KzPrefsWinPagePriv, 1);
 			priv->entry  = entry;
 			priv->widget = NULL;
-			win->priv->page_list = g_list_append(win->priv->page_list, priv);
+			winpriv->page_list = g_list_append(winpriv->page_list,
+							   priv);
 		}
 
 		if (!(priv->entry->ui_level & kz_ui_level()))
@@ -515,8 +529,8 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 			priv->widget = NULL;
 			continue;
 		}
-#if 0
-		if (kz_prefs_win_ensure_parent (priv, &parent_iter))
+
+		if (kz_prefs_win_ensure_parent (win, priv, &parent_iter))
 		{
 			gtk_tree_store_append (store, &iter, &parent_iter);
 		}
@@ -524,23 +538,23 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 		{
 			gtk_tree_store_append (store, &iter, NULL);
 		}
-#else
-		gtk_tree_store_append (store, &iter, NULL);
-#endif
 
 		title = g_path_get_basename(_(entry->path));
 		gtk_tree_store_set(store, &iter,
-#if 0
+				   /*
 				   COLUMN_ICON_CLOSE,      pixbuf,
 				   COLUMN_ICON_OPEN,       pixbuf_open,
-#endif
+				   */
 				   COLUMN_NAME,            title,
 				   COLUMN_PRIV_DATA,       priv,
 				   COLUMN_TERMINATOR);
 		g_free(title);
 	}
 
-	/* FIXME: set cursor */
+	if (path) {
+		gtk_tree_view_set_cursor(treeview, path, NULL, FALSE);
+		gtk_tree_path_free(path);
+	}
 }
 
 static void
