@@ -729,13 +729,7 @@ kz_moz_embed_view_source (KzEmbed *kzembed, const gchar *url)
 	NewURI(getter_AddRefs(aURI), uri);
 	/* FIXME! set correct contentType */
 	nsEmbedCString contentType;
-	entry->Create(aURI,
-		      nsnull,
-		      nsnull,
-		      nsnull,
-		      nsnull,
-		      nsnull,
-		      nsEmbedCString(""));
+	entry->SetURI(aURI);
 	sHistoryInternal->AddEntry(entry, PR_TRUE);
 	g_free(uri);
 }
@@ -2919,13 +2913,7 @@ kz_moz_embed_set_history (KzEmbed *kzembed, KzBookmark *history)
 		NewURI(getter_AddRefs(aURI), uri);		
 		/* FIXME! set correct contentType */
 		nsEmbedCString contentType;
-		entry->Create(aURI,
-			      nsnull,
-			      nsnull,
-			      nsnull,
-			      nsnull,
-			      nsnull,
-			      nsEmbedCString(""));
+		entry->SetURI(aURI);
 		sHistoryInternal->AddEntry(entry, PR_TRUE);
 	}
 	g_list_free(children);
@@ -2962,7 +2950,7 @@ kz_moz_embed_get_history (KzEmbed *kzembed, KzBookmark *history)
 	sHistory->GetIndex(&index);
 	for (PRInt32 i = 0; i < count; i++)
 	{
-		gchar *title, *uri;
+		gchar *title = NULL, *uri = NULL;
 		KzBookmark *bookmark = NULL;
 		
 		kz_moz_embed_shistory_get_nth (kzembed,
@@ -2982,8 +2970,9 @@ kz_moz_embed_get_history (KzEmbed *kzembed, KzBookmark *history)
 			kz_bookmark_append(history, bookmark);
 			g_object_unref(bookmark);
 		}
-		else if (!strcmp(uri, kz_bookmark_get_link(bookmark)))
+		else if (uri && !strcmp(uri, kz_bookmark_get_link(bookmark)))
 		{
+			g_free(uri);
 			continue;
 		}
 		else
