@@ -23,12 +23,7 @@
 
 #include "kz-mozeventlistener.h"
 
-#include <nsCOMPtr.h>
 #include <nsIDOMNode.h>
-#include <nsIDOMElement.h>
-#define MOZILLA_STRICT_API
-#include <nsEmbedString.h>
-#undef MOZILLA_STRICT_API
 #include <nsIDOMDocument.h>
 #include <nsIURI.h>
 #include <nsIDOMEventTarget.h>
@@ -67,20 +62,11 @@ KzMozEventListener::HandleLink (nsIDOMNode *node)
 	if (!linkElement) return NS_ERROR_FAILURE;
 
 	nsEmbedString name;
-
-	nsEmbedCString crel("rel");
-	nsEmbedString rel;
-	NS_CStringToUTF16(crel, NS_CSTRING_ENCODING_UTF8, rel);
-
-	result = linkElement->GetAttribute (rel, name);
+	result = GetLinkAttribute(linkElement, "rel", &name);
 	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
 
 	nsEmbedString link;
-	nsEmbedCString chref("href");
-	nsEmbedString href;
-	NS_CStringToUTF16(chref, NS_CSTRING_ENCODING_UTF8, href);
-
-	result = linkElement->GetAttribute (href, link);
+	result = GetLinkAttribute(linkElement, "href", &link);
 	if (NS_FAILED (result) || !link.Length()) return NS_ERROR_FAILURE;
 
 	nsCOMPtr<nsIDOMDocument> domDoc;
@@ -106,6 +92,13 @@ KzMozEventListener::HandleLink (nsIDOMNode *node)
 	result = baseURI->Resolve (linkstring, url);
 	if (NS_FAILED (result)) return NS_ERROR_FAILURE;
 
+	nsEmbedString type;
+	result = GetLinkAttribute(linkElement, "type", &type);
+	if (NS_FAILED(result)) return NS_ERROR_FAILURE;
+        
+	nsEmbedCString cType;
+	NS_UTF16ToCString(type, NS_CSTRING_ENCODING_UTF8, cType);
+        
 	nsEmbedCString cName;
 	NS_UTF16ToCString(name, NS_CSTRING_ENCODING_UTF8, cName);
 	if (!g_ascii_strcasecmp(cName.get(),"SHORTCUT ICON") ||
@@ -118,11 +111,14 @@ KzMozEventListener::HandleLink (nsIDOMNode *node)
 		g_free (favicon_uri);
 		g_object_unref(kzfav);
 	}
+	else if (!g_ascii_strcasecmp(cName.get(), "ALTERNATE") &&
+		 !g_ascii_strcasecmp(cType.get(), "application/rss+xml"))
+	{
+		kz_embed_set_nav_link(KZ_EMBED(mOwner), "RSS", url.get());
+	}
 	else //if (link.EqualsIgnoreCase("ALTERNATE"))
 	{
-		kz_embed_set_nav_link(KZ_EMBED(mOwner),
-				      cName.get(),
-				      cName.get());
+		kz_embed_set_nav_link(KZ_EMBED(mOwner), cName.get(), url.get());
 	}
 		
 	return NS_OK;
@@ -143,3 +139,15 @@ KzMozEventListener::HandleEvent(nsIDOMEvent* aDOMEvent)
 
 	return NS_OK;
 }
+
+nsresult
+KzMozEventListener::GetLinkAttribute (nsCOMPtr<nsIDOMElement>& linkElement,
+				      const char *name,
+				      nsEmbedString *value)
+{
+	nsEmbedString n_name;
+	nsEmbedCString c_name(name);
+	NS_CStringToUTF16(c_name, NS_CSTRING_ENCODING_UTF8, n_name);
+	
+	return linkElement->GetAttribute(n_name, *value);
+}
