@@ -31,8 +31,6 @@
 #include <nsIURI.h>
 #include <nsIIOService.h>
 #include <nsCRT.h>
-#include <nsIComponentManager.h>
-#include <nsIServiceManager.h>
 #include <nsIStorageStream.h>
 #include <nsIOutputStream.h>
 #define MOZILLA_STRICT_API
@@ -40,6 +38,7 @@
 #undef MOZILLA_STRICT_API
 #include <nsNetCID.h>
 #include <nsILocalFile.h>
+#include <nsMemory.h>
 
 #include <glib.h>
 
