@@ -30,8 +30,10 @@
 #include <nsCOMPtr.h>
 #include <nsIPrefService.h>
 #include <nsISupportsPrimitives.h>
+#define MOZILLA_INTERNAL_API
 #include <nsIServiceManager.h>
-#if 0
+#undef MOZILLA_INTERNAL_API
+#if 1
 #include <nsIFontEnumerator.h>
 #endif
 #include <nsISimpleEnumerator.h>
@@ -293,6 +295,7 @@ mozilla_prefs_get_font_list (const char *lang_group,
 			     GList **all_font_list,
 			     char **default_font)
 {
+#if 0
 	nsresult rv;
 
 	nsCOMPtr<nsIFontList> mozFontList;
@@ -354,8 +357,9 @@ mozilla_prefs_get_font_list (const char *lang_group,
 		*default_font = g_strdup(value);
 		nsMemory::Free(value);
 	}
+#endif
 
-#if 0
+#if 1
 	nsresult rv;
 	PRUint32 count;
 	PRUnichar **fontArray;
@@ -401,9 +405,13 @@ mozilla_prefs_get_font_list (const char *lang_group,
 			nsEmbedCString tmp;
 			NS_UTF16ToCString(nsEmbedString(fontArray[i]),
 					  NS_CSTRING_ENCODING_UTF8, tmp);
-			gFontString = g_strdup(tmp.get());
-			list = g_list_prepend(list, gFontString);
-			nsMemory::Free (fontArray[i]);
+			if (!g_list_find_custom(*font_list, tmp.get(),
+					        (GCompareFunc)strcmp))
+			{
+				gFontString = g_strdup(tmp.get());
+				list = g_list_prepend(list, gFontString);
+				nsMemory::Free (fontArray[i]);
+			}
 		}
 		*all_font_list = g_list_reverse(list);
 	}
@@ -411,6 +419,7 @@ mozilla_prefs_get_font_list (const char *lang_group,
 
 	if (default_font != NULL)
 	{
+#if 0
 		PRUnichar *value;
 		rv = mozFontEnumerator->GetDefaultFont(lang_group,
 						       font_type,
@@ -420,6 +429,24 @@ mozilla_prefs_get_font_list (const char *lang_group,
 				  NS_CSTRING_ENCODING_UTF8, tmp);
 		*default_font = g_strdup(tmp.get());
 		nsMemory::Free(value);
+#endif		
+		char key[255];
+		char *value = NULL;
+		nsCOMPtr<nsIPrefService> prefService;
+
+	        prefService = do_GetService(NS_PREFSERVICE_CONTRACTID);
+		g_return_val_if_fail(prefService != NULL, FALSE);
+	
+	        nsCOMPtr<nsIPrefBranch> pref;
+	        prefService->GetBranch("", getter_AddRefs(pref));
+		g_return_val_if_fail(pref != NULL, FALSE);
+
+		g_snprintf(key, sizeof(key), "font.name.%s.%s",
+			   font_type, lang_group);
+		
+		pref->GetCharPref(key, &value);
+		*default_font = g_strdup(value);
+		nsMemory::Free(value);
 	}
 #endif
 	return TRUE;
