@@ -189,6 +189,23 @@ compare_font_pref (gconstpointer a, gconstpointer b)
 }
 
 
+static gboolean
+combo_box_row_separator_func (GtkTreeModel *model,
+			      GtkTreeIter  *iter,
+			      gpointer      user_data)
+{
+	gchar *text;
+	gboolean ret;
+
+	gtk_tree_model_get (model, iter, 0, &text, -1);
+
+	ret = (text && !strncmp(text, "---", 3));
+
+	if (text)
+		g_free(text);
+	return ret;
+}			  
+
 static void
 font_combo_set (GtkComboBox *combo,
 		const gchar *lang, const gchar *type,
@@ -220,6 +237,20 @@ font_combo_set (GtkComboBox *combo,
 		i++;
 	}
 
+#if GTK_CHECK_VERSION(2,6,0)
+	/* for separator */
+	gtk_combo_box_append_text(combo, "---");
+	i++;
+
+	for (node = g_list_first(all_fonts); node; node = g_list_next(node))
+	{
+		if (use_font && node->data && !strcmp(use_font, node->data))
+			select = i;
+		gtk_combo_box_append_text(combo, node->data);	
+		i++;
+	}
+#endif
+
 	gtk_combo_box_set_active(combo, select);
 	
 	g_free (default_font);
@@ -603,6 +634,11 @@ prefs_font_create (void)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
 	g_signal_connect (combo, "changed",
 			  G_CALLBACK(cb_serif_entry_changed), prefsui);
+#if GTK_CHECK_VERSION(2,6,0)
+	gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combo),
+					     combo_box_row_separator_func,
+					     NULL, NULL);
+#endif
 	gtk_table_attach(GTK_TABLE(table), combo,
 			 1, 2, 1, 2,
 			 GTK_EXPAND | GTK_FILL, 0,
@@ -629,6 +665,11 @@ prefs_font_create (void)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
 	g_signal_connect (combo, "changed",
 			  G_CALLBACK(cb_sans_serif_entry_changed), prefsui);
+#if GTK_CHECK_VERSION(2,6,0)
+	gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combo),
+					     combo_box_row_separator_func,
+					     NULL, NULL);
+#endif
 	gtk_table_attach(GTK_TABLE(table), combo,
 			 1, 2, 2, 3,
 			 GTK_EXPAND | GTK_FILL, 0,
@@ -655,6 +696,11 @@ prefs_font_create (void)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label), combo);
 	g_signal_connect (combo, "changed",
 			  G_CALLBACK(cb_monospace_entry_changed), prefsui);
+#if GTK_CHECK_VERSION(2,6,0)
+	gtk_combo_box_set_row_separator_func(GTK_COMBO_BOX(combo),
+					     combo_box_row_separator_func,
+					     NULL, NULL);
+#endif
 	gtk_table_attach(GTK_TABLE(table), combo,
 			 1, 2, 3, 4,
 			 GTK_EXPAND | GTK_FILL, 0,
