@@ -31,8 +31,11 @@
 
 
 static GtkWidget     *prefs_tab_create          (void);
+static GtkWidget     *prefs_new_tab_create      (void);
 static void           prefs_tab_response        (GtkWidget *widget,
 						 gint       response);
+static void           prefs_new_tab_response    (GtkWidget *widget,
+						 gint       response);
 
 
 static KzPrefsWinPageEntry prefs_entry =
@@ -44,6 +47,15 @@ static KzPrefsWinPageEntry prefs_entry =
 	response:        prefs_tab_response,
 };
 
+static KzPrefsWinPageEntry prefs_new_tab_entry =
+{
+	path:            N_("/Tab/New tab"),
+	priority_hint:   0,
+	ui_level:        ~KZ_UI_LEVEL_BEGINNER,
+	create:          prefs_new_tab_create,
+	response:        prefs_new_tab_response,
+};
+
 
 typedef struct _KzPrefsTab
 {
@@ -54,13 +66,19 @@ typedef struct _KzPrefsTab
 	GtkWidget     *show_close_button;
 	GtkWidget     *wheel_circulation;
 	GtkComboBox   *close_menu;
+	GdkColor       color[KZ_TAB_LABEL_N_STATE];
+	gboolean       changed;
+} KzPrefsTab;
+
+typedef struct _KzPrefsNewTab
+{
+	GtkWidget     *main_vbox;
    	GtkWidget     *focus_new_tab;
    	GtkWidget     *focus_loc_ent;
    	GtkWidget     *focus_mid_click_link;
 	GtkComboBox   *new_tab_position_menu;
-	GdkColor       color[KZ_TAB_LABEL_N_STATE];
 	gboolean       changed;
-} KzPrefsTab;
+} KzPrefsNewTab;
 
 
 typedef struct
@@ -139,7 +157,7 @@ prefs_tab_create (void)
 	GtkWidget *label, *check, *spin, *option_menu;
 	GtkWidget *cbutton;
 	GtkAdjustment *adj;
-	gboolean fix, show, focus, key_focus, circulation;
+	gboolean fix, show, circulation;
 	gint width;
 	gchar *str;
 
@@ -286,80 +304,6 @@ prefs_tab_create (void)
 	g_signal_connect (option_menu, "changed",
 			  G_CALLBACK (cb_changed), prefsui);
 
-	/*
-	 *  New Tab
-	 */
-	frame = gtk_frame_new(_("New Tab"));
-	gtk_frame_set_label_align(GTK_FRAME(frame), 0.03, 0.5);
-	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
-	gtk_box_pack_start(GTK_BOX(main_vbox), frame, FALSE, FALSE, 2);
-	gtk_widget_show(frame);
-
-	vbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
-	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_widget_show(vbox);
-
-	check = gtk_check_button_new_with_mnemonic(_("_Move focus to new tab on NewTab Action"));
-	prefsui->focus_new_tab = check;
-	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
-	focus = FALSE;
-	KZ_CONF_GET("Tab", "focus_open_new", focus, BOOL);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), focus);
-	g_signal_connect(check, "toggled",
-			 G_CALLBACK(cb_changed), prefsui);
-	gtk_widget_show(check);
-
-	check = gtk_check_button_new_with_mnemonic(_("Move focus to new tab on middle-clicking link"));
-	prefsui->focus_mid_click_link = check;
-	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
-	focus = FALSE;
-	KZ_CONF_GET("Tab", "focus_mid_click_link", focus, BOOL);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), focus);
-	g_signal_connect(check, "toggled",
-			 G_CALLBACK(cb_changed), prefsui);
-	gtk_widget_show(check);
-
-	check = gtk_check_button_new_with_mnemonic(_("Move _Keyboad focus to location entry when swich to new empty tab"));
-	prefsui->focus_loc_ent = check;
-	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
-	key_focus = FALSE;
-	KZ_CONF_GET("Tab", "focus_loc_ent_new", key_focus, BOOL);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), key_focus);
-	g_signal_connect(check, "toggled",
-			 G_CALLBACK(cb_changed), prefsui);
-	gtk_widget_show(check);
-	
-	/*
-	 *  New tab positon
-	 */
-	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
-	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
-	gtk_widget_show(hbox);
-
-	label = gtk_label_new_with_mnemonic(_("_New Tab Position: "));
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
-	gtk_widget_show(label);
-
-	option_menu = gtk_combo_box_new_text();
-	prefsui->new_tab_position_menu = GTK_COMBO_BOX(option_menu);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), option_menu);
-	gtk_box_pack_start(GTK_BOX(hbox), option_menu, FALSE, FALSE, 0);
-	gtk_widget_show(option_menu);
-
-	GTKUTIL_CREATE_MENU(option_menu,
-			    new_tab_position,
-			    n_new_tab_position,
-			    title);
-	str = KZ_CONF_GET_STR("Tab", "new_tab_position");
-	GTKUTIL_SET_DEFAULT_HISTORY(option_menu, str,
-				    new_tab_position, 
-				    n_new_tab_position, code);
-	g_free(str);
-	g_signal_connect (option_menu, "changed",
-			  G_CALLBACK (cb_changed), prefsui);
-
 	/*
 	 * Tab color selection
 	 */
@@ -466,6 +410,92 @@ prefs_tab_create (void)
 	return main_vbox;
 }
 
+static GtkWidget *
+prefs_new_tab_create (void)
+{
+	KzPrefsNewTab *prefsui = g_new0(KzPrefsNewTab, 1);
+	GtkWidget *main_vbox, *vbox, *hbox;
+	GtkWidget *label, *check, *option_menu;
+	gboolean focus, key_focus;
+	gchar *str;
+
+	main_vbox = gtk_vbox_new(FALSE, 0);
+	prefsui->main_vbox = main_vbox;
+	g_object_set_data_full(G_OBJECT(main_vbox), DATA_KEY,
+			       prefsui, prefs_tab_destroy);
+
+	label = kz_prefs_ui_utils_create_title(_("New tab"));
+	gtk_box_pack_start(GTK_BOX(main_vbox), label,
+			   FALSE, FALSE, 0);
+	gtk_widget_show(label);
+
+	vbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
+	gtk_box_pack_start(GTK_BOX(main_vbox), vbox, FALSE, FALSE, 0);
+	gtk_widget_show(vbox);
+
+	check = gtk_check_button_new_with_mnemonic(_("_Move focus to new tab on NewTab Action"));
+	prefsui->focus_new_tab = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	focus = FALSE;
+	KZ_CONF_GET("Tab", "focus_open_new", focus, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), focus);
+	g_signal_connect(check, "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+
+	check = gtk_check_button_new_with_mnemonic(_("Move focus to new tab on middle-clicking link"));
+	prefsui->focus_mid_click_link = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	focus = FALSE;
+	KZ_CONF_GET("Tab", "focus_mid_click_link", focus, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), focus);
+	g_signal_connect(check, "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+
+	check = gtk_check_button_new_with_mnemonic(_("Move _Keyboad focus to location entry when swich to new empty tab"));
+	prefsui->focus_loc_ent = check;
+	gtk_box_pack_start(GTK_BOX(vbox), check, FALSE, FALSE, 2);
+	key_focus = FALSE;
+	KZ_CONF_GET("Tab", "focus_loc_ent_new", key_focus, BOOL);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), key_focus);
+	g_signal_connect(check, "toggled",
+			 G_CALLBACK(cb_changed), prefsui);
+	gtk_widget_show(check);
+	
+	/*
+	 *  New tab positon
+	 */
+	hbox = gtk_hbox_new(FALSE, 0);
+	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 2);
+	gtk_widget_show(hbox);
+
+	label = gtk_label_new_with_mnemonic(_("_New Tab Position: "));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 2);
+	gtk_widget_show(label);
+
+	option_menu = gtk_combo_box_new_text();
+	prefsui->new_tab_position_menu = GTK_COMBO_BOX(option_menu);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), option_menu);
+	gtk_box_pack_start(GTK_BOX(hbox), option_menu, FALSE, FALSE, 0);
+	gtk_widget_show(option_menu);
+
+	GTKUTIL_CREATE_MENU(option_menu,
+			    new_tab_position,
+			    n_new_tab_position,
+			    title);
+	str = KZ_CONF_GET_STR("Tab", "new_tab_position");
+	GTKUTIL_SET_DEFAULT_HISTORY(option_menu, str,
+				    new_tab_position, 
+				    n_new_tab_position, code);
+	g_free(str);
+	g_signal_connect (option_menu, "changed",
+			  G_CALLBACK (cb_changed), prefsui);
+
+	return main_vbox;
+}
 
 static void
 prefs_tab_response (GtkWidget *widget, gint response)
@@ -484,7 +514,6 @@ prefs_tab_response (GtkWidget *widget, gint response)
 		gint width;
 		gboolean fix_width;
 		gboolean show_button, show_favicon;
-		gboolean focus_new_tab, focus_loc_ent, focus_mid_click_link;
 		gboolean circulation;
 		
 		if (!prefsui->changed)
@@ -516,23 +545,6 @@ prefs_tab_response (GtkWidget *widget, gint response)
 			KZ_CONF_SET_STR("Tab", "page_to_return_when_close",
 					page_to_return[i].code);
 
-		focus_new_tab = gtk_toggle_button_get_active
-				(GTK_TOGGLE_BUTTON(prefsui->focus_new_tab));
-		KZ_CONF_SET("Tab", "focus_open_new", focus_new_tab, BOOL);
-
-		focus_mid_click_link = gtk_toggle_button_get_active
-				(GTK_TOGGLE_BUTTON(prefsui->focus_mid_click_link));
-		KZ_CONF_SET("Tab", "focus_mid_click_link", focus_mid_click_link, BOOL);
-
-		focus_loc_ent = gtk_toggle_button_get_active
-				(GTK_TOGGLE_BUTTON(prefsui->focus_loc_ent));
-		KZ_CONF_SET("Tab", "focus_loc_ent_new", focus_loc_ent, BOOL);
-
-		i = gtk_combo_box_get_active(prefsui->new_tab_position_menu);
-		if (i >= 0 && i < n_new_tab_position)
-			KZ_CONF_SET_STR("Tab", "new_tab_position",
-					new_tab_position[i].code);
-
 		g_snprintf(str_color, G_N_ELEMENTS(str_color),
 			   "#%04X%04X%04X" ,
 			   prefsui->color[KZ_TAB_LABEL_STATE_NORMAL].red,
@@ -571,12 +583,65 @@ prefs_tab_response (GtkWidget *widget, gint response)
 	}
 }
 
+static void
+prefs_new_tab_response (GtkWidget *widget, gint response)
+{
+	KzPrefsNewTab *prefsui = g_object_get_data(G_OBJECT(widget), DATA_KEY);
+
+	g_return_if_fail(prefsui);
+
+	switch (response) {
+	case GTK_RESPONSE_ACCEPT:
+	case GTK_RESPONSE_APPLY:
+	{
+		guint i;
+		gboolean focus_new_tab, focus_loc_ent, focus_mid_click_link;
+		
+		if (!prefsui->changed)
+			break;
+
+		focus_new_tab = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(prefsui->focus_new_tab));
+		KZ_CONF_SET("Tab", "focus_open_new", focus_new_tab, BOOL);
+
+		focus_mid_click_link = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(prefsui->focus_mid_click_link));
+		KZ_CONF_SET("Tab", "focus_mid_click_link", focus_mid_click_link, BOOL);
+
+		focus_loc_ent = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON(prefsui->focus_loc_ent));
+		KZ_CONF_SET("Tab", "focus_loc_ent_new", focus_loc_ent, BOOL);
+
+		i = gtk_combo_box_get_active(prefsui->new_tab_position_menu);
+		if (i >= 0 && i < n_new_tab_position)
+			KZ_CONF_SET_STR("Tab", "new_tab_position",
+					new_tab_position[i].code);
+
+		prefsui->changed = FALSE;
+
+		break;
+	}
+	case GTK_RESPONSE_REJECT:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_MEDIUM:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_EXPERT:
+		break;
+	case KZ_RESPONSE_UI_LEVEL_CUSTOM:
+		break;
+	default:
+		break;
+	}
+}
+
 
 KzPrefsWinPageEntry *
 prefs_tab_get_entry (gint idx)
 {
 	if (idx == 0)
 		return &prefs_entry;
+	else if (idx == 1)
+		return &prefs_new_tab_entry;
 	else
 		return NULL;
 }
